/** Kerrighed tools user interface.
 *  @file libkerrighed_tools.h
 *
 *  @author David Margery, Renaud Lottiaux
 *  @author Pascal Gallard (Kerlabs)
 */

#ifndef LIBKERRIGHED_TOOL_H

#define LIBKERRIGHED_TOOL_H

#define KERRIGHED_PROC_MAGIC 0xD1

#define TOOLS_PROC_BASE 0
#define COMM_PROC_BASE 32
#define SYNC_PROC_BASE 64
#define KERMM_PROC_BASE 96
#define KERPROC_PROC_BASE 128
#define EPM_PROC_BASE 192

/*
 * Tools related Kerrighed syscalls
 */

#define KSYS_SET_CAP          _IOW(KERRIGHED_PROC_MAGIC, \
                                   TOOLS_PROC_BASE + 0, \
                                   krg_cap_t )

#define KSYS_GET_CAP          _IOR(KERRIGHED_PROC_MAGIC, \
                                   TOOLS_PROC_BASE + 1, \
                                   krg_cap_t )

#define KSYS_SET_PID_CAP      _IOW(KERRIGHED_PROC_MAGIC, \
                                   TOOLS_PROC_BASE + 2, \
                                   krg_cap_pid_t )

#define KSYS_GET_PID_CAP      _IOR(KERRIGHED_PROC_MAGIC, \
                                   TOOLS_PROC_BASE + 3, \
                                   krg_cap_pid_t )

#define KSYS_SET_FATHER_CAP   _IOW(KERRIGHED_PROC_MAGIC, \
                                   TOOLS_PROC_BASE + 4, \
                                   krg_cap_t )

#define KSYS_GET_FATHER_CAP   _IOR(KERRIGHED_PROC_MAGIC, \
                                   TOOLS_PROC_BASE + 5, \
                                   krg_cap_t )

#define KSYS_NB_MAX_NODES     _IOR(KERRIGHED_PROC_MAGIC, \
                                   TOOLS_PROC_BASE + 6,  \
				   int)
#define KSYS_NB_MAX_CLUSTERS  _IOR(KERRIGHED_PROC_MAGIC, \
                                   TOOLS_PROC_BASE + 7,  \
				   int)

/*
 * Communications related Kerrighed syscalls
 */

#define KSYS_GET_NODE_ID       _IOR(KERRIGHED_PROC_MAGIC, \
				    COMM_PROC_BASE + 0, \
                                    int)
#define KSYS_GET_NODES_COUNT   _IOR(KERRIGHED_PROC_MAGIC, \
				    COMM_PROC_BASE + 1,   \
                                    int)
#define KSYS_COMM_PING_PONG    _IO(KERRIGHED_PROC_MAGIC,  \
				    COMM_PROC_BASE + 2)
#define KSYS_HOTPLUG_START     _IOW(KERRIGHED_PROC_MAGIC, \
                                    COMM_PROC_BASE + 3,   \
                                    krgnodemask_t)
#define KSYS_HOTPLUG_RESTART   _IOW(KERRIGHED_PROC_MAGIC, \
                                    COMM_PROC_BASE + 4,   \
                                    krgnodemask_t)
#define KSYS_HOTPLUG_SHUTDOWN  _IOW(KERRIGHED_PROC_MAGIC, \
                                    COMM_PROC_BASE + 5,   \
                                    krgnodemask_t)
#define KSYS_HOTPLUG_REBOOT    _IOW(KERRIGHED_PROC_MAGIC, \
                                    COMM_PROC_BASE + 6,   \
                                    krgnodemask_t)
#define KSYS_HOTPLUG_STATUS    _IOR(KERRIGHED_PROC_MAGIC, \
                                    COMM_PROC_BASE + 7,   \
				    struct hotplug_clusters)
#define KSYS_HOTPLUG_ADD       _IOW(KERRIGHED_PROC_MAGIC, \
                                    COMM_PROC_BASE + 8,   \
                                    struct hotplug_node_set)
#define KSYS_HOTPLUG_REMOVE    _IOW(KERRIGHED_PROC_MAGIC, \
                                    COMM_PROC_BASE + 9,   \
                                    struct hotplug_node_set)
#define KSYS_HOTPLUG_FAIL      _IOW(KERRIGHED_PROC_MAGIC, \
                                    COMM_PROC_BASE + 10,   \
                                    struct hotplug_node_set)
#define KSYS_HOTPLUG_NODES     _IOWR(KERRIGHED_PROC_MAGIC, \
                                     COMM_PROC_BASE + 11,  \
				     struct hotplug_nodes)
#define KSYS_HOTPLUG_POWEROFF  _IOW(KERRIGHED_PROC_MAGIC, \
                                    COMM_PROC_BASE + 12,  \
				    struct hotplug_node_set)

/*
 * Synchronization related Kerrighed syscalls
 */

#define KSYS_CREATE_GROUP    _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 0, \
                                  struct krg_group_info)

#define KSYS_FREE_GROUP      _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 21, \
                                  struct krg_group_info)

#define KSYS_JOIN_GROUP      _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 1, \
                                  struct krg_group_info)

#define KSYS_LEAVE_GROUP     _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 2, \
                                  struct krg_group_info)

#define KSYS_ENTER_BARRIER   _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 3, \
                                  struct krg_group_info)

#define KSYS_CREATE_LOCK     _IOR(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 4, \
                                  unsigned long)

#define KSYS_FREE_LOCK       _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 5, \
                                  unsigned long)

#define KSYS_LOCK            _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 6, \
                                  unsigned long)

#define KSYS_TRYLOCK         _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 7, \
                                  unsigned long)

#define KSYS_UNLOCK          _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 8, \
                                  unsigned long)

#define KSYS_CREATE_COND     _IOR(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 9, \
                                  unsigned long)

#define KSYS_FREE_COND       _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 10, \
                                  unsigned long)

#define KSYS_COND_WAIT       _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 11, \
                                  unsigned long)

#define KSYS_COND_SIGNAL     _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 12, \
                                  unsigned long)

#define KSYS_COND_BROADCAST  _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 13, \
                                  unsigned long)

#define KSYS_COND_TIMEDWAIT  _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 14, \
                                  unsigned long)

#define KSYS_CREATE_SEM      _IOR(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 15, \
                                  unsigned long)

#define KSYS_FREE_SEM        _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 16, \
                                  unsigned long)

#define KSYS_SEM_WAIT        _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 17, \
                                  unsigned long)

#define KSYS_SEM_TRYWAIT     _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 18, \
                                  unsigned long)

#define KSYS_SEM_POST        _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 19, \
                                  unsigned long)

#define KSYS_SEM_GETVALUE    _IOW(KERRIGHED_PROC_MAGIC, \
                                  SYNC_PROC_BASE + 20, \
                                  unsigned long)

/*
 *  Memory related Kerrighed syscalls
 */

#define KSYS_CHANGE_MAP_LOCAL_VALUE  _IOW(KERRIGHED_PROC_MAGIC, \
			                  KERMM_PROC_BASE + 0, \
					   struct kermm_new_local_data)

#define KSYS_PREFETCH_MEMORY         _IOWR(KERRIGHED_PROC_MAGIC, \
				           KERMM_PROC_BASE + 1,\
					   gandalf_prefetch_memory_data_t)

/*
 * Process Management related kerrighed syscalls
 */

#define KSYS_GET_KRG_THREAD_ID               _IOW(KERRIGHED_PROC_MAGIC, \
                                                  KERPROC_PROC_BASE + 0, \
                                                  int)
#define KSYS_KRG_THREAD_EXIT                 _IOR(KERRIGHED_PROC_MAGIC, \
                                                  KERPROC_PROC_BASE + 1, \
                                                  int)
#define KSYS_WAIT_KRG_THREAD_END             _IOWR(KERRIGHED_PROC_MAGIC, \
                                                   KERPROC_PROC_BASE + 2, \
                                                   int)
#define KSYS_KRG_THREAD_DETACH               _IOWR(KERRIGHED_PROC_MAGIC, \
                                                   KERPROC_PROC_BASE + 3, \
                                                   int)
#define KSYS_KILL_KRG_THREAD                 _IOW(KERRIGHED_PROC_MAGIC, \
                                                  KERPROC_PROC_BASE + 4, \
                                                  unsigned long)
#define KSYS_ADD_PROCESS_IN_PROC_GROUP       _IOR(KERRIGHED_PROC_MAGIC, \
                                                  KERPROC_PROC_BASE + 5, \
                                                  process_group_add_info_t)
#define KSYS_REMOVE_PROCESS_FROM_PROC_GROUP  _IOR(KERRIGHED_PROC_MAGIC, \
                                                  KERPROC_PROC_BASE + 6, \
                                                  process_group_add_info_t)
#define KSYS_ADD_NODE_IN_PROC_GROUP          _IOR(KERRIGHED_PROC_MAGIC, \
                                                  KERPROC_PROC_BASE + 7, \
                                                  node_group_add_info_t)
#define KSYS_REMOVE_NODE_FROM_PROC_GROUP     _IOR(KERRIGHED_PROC_MAGIC, \
                                                  KERPROC_PROC_BASE + 8, \
                                                  node_group_add_info_t)
#define KSYS_CREATE_PROC_GROUP               _IOR(KERRIGHED_PROC_MAGIC, \
		                                  KERPROC_PROC_BASE + 9, \
		                                  process_group_id_t)
#define KSYS_GET_PROC_GROUP                  _IOR(KERRIGHED_PROC_MAGIC, \
		                                  KERPROC_PROC_BASE + 10, \
		                                  process_group_id_t)
#define KSYS_DELETE_PROC_GROUP               _IOW(KERRIGHED_PROC_MAGIC, \
		                                  KERPROC_PROC_BASE + 11, \
		                                  process_group_id_t)

/*
 * Enhanced Process Management related kerrighed syscalls
 */

#define KSYS_PROCESS_MIGRATION         _IOW(KERRIGHED_PROC_MAGIC, \
                                            EPM_PROC_BASE + 0, \
                                            migration_infos_t)
#define KSYS_THREAD_MIGRATION	       _IOW(KERRIGHED_PROC_MAGIC, \
                                            EPM_PROC_BASE + 1,\
                                            migration_infos_t)
#define KSYS_APP_CHKPT                 _IOW(KERRIGHED_PROC_MAGIC, \
                                            EPM_PROC_BASE + 2, \
                                            checkpoint_infos_t)
#define KSYS_APP_RESTART               _IOW(KERRIGHED_PROC_MAGIC, \
                                            EPM_PROC_BASE + 3, \
                                            restart_request_t)

/*
 *
 * Additionnal types
 *
 */

/*
 * Comm
 */

struct proc_gimli_stats {
	int size;
	int loops;
	int partner;
	unsigned long min;
	unsigned long max;
	unsigned long avg;
};

/*
 * HotPlug
 */

struct hotplug_nodes {
	char *nodes;
};

struct hotplug_clusters {
	char clusters[KERRIGHED_MAX_CLUSTERS];
};

/*
  Warning: krgnodemask_t from user space is different from krgnodemask_t from
  kernel space. In user space, the size of the vector is related to
  KERRIGHED_HARD_MAX_NODES (and not to KERRIGHED_MAX_NODES)
*/
struct hotplug_node_set {
	int subclusterid;
	krgnodemask_t v;
};

/*
 * Sync
 */

#define KERRIGHED_BARRIER_ELECTED 1

/* Lock kinds */
#define REGULAR_LOCK 1
#define RECURSIVE_LOCK 2

struct krg_group_info {
	char *key;
	int group_id;
	int nr_process;
};

struct krg_lock_info {
	char *key;
	int lock_id;
	int lock_kind;
};

struct krg_cond_info {
	char *key;
	int cond_id;
	int lock_id;
	struct timespec timeout;
};

struct krg_sem_info {
	char *key;
	int sem_id;
	int value;
	struct timespec timeout;
};

/*
 * Ctnr
 */
#define MEMORY_PREFETCH_READ 1
#define MEMORY_PREFETCH_WRITE 2

typedef struct kermm_new_local_data {
	unsigned long data_place;
} kermm_new_local_data_t;

typedef struct gandalf_prefetch_memory_data {
	void *start;
	size_t length;
	int prefetch_type;
} gandalf_prefetch_memory_data_t;

/** Call a kerrighed service.
 *  @author Renaud Lottiaux
 *
 * This is the easy function to use, as it opens kerrighed services, calls
 * the service and closes kerrighed services .
 * The drawback of this function is that it involve three system calls each
 * time it is called, hence the other variant
 */
int call_kerrighed_services(int service_id, void * data) ;



/** Open kerrighed services
 *  @author David Margery
 *
 *  @return the file descriptor of the openned kerrighed service, -1 if failure
 */
int open_kerrighed_services() ;



/** Close kerrighed services
 *  @author David Margery
 *
 *  @param fd : the file descriptor returned when the kerrighed services
 *              where opened
 */
void close_kerrighed_services(int fd) ;



/** Call a kerrighed service that was allready opened
 *  @author David Margery
 *
 *  @param fd : the file descriptor returned when the kerrighed services
 *              where opened
 *  @param service_id : the service called
 *  @param data : the data needed by the service
 */
int call_opened_kerrighed_services(int fd, int service_id, void * data) ;

#endif // LIBKERRIGHED_TOOL_H
